#ifndef Trans_hh
#define Trans_hh


/*
 * Copyright 1995 by Arvin Schnell
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */


#include <minmax.h>


/*
 * Transformation zwischen zwei 1.dim. Koordinatensystemen.
 */


template<class TA, class TB> class Trans1 {
  
public:
  
  Trans1();
  void clear();
  
  void setA( const TA, const TA );
  void setB( const TB, const TB );
  
  void AtoB( const TA, TB & ) const;
  void BtoA( const TB, TA & ) const;
  
  int inA( const TA ) const;
  int inB( const TB ) const;
  
  void calc();
  
private:
  
  TA A1, A2;
  TB B1, B2;
  
  double Am, Ac;
  double Bm, Bc;
  
  int flagA, flagB;
  
};



/*
 * Transformation zwischen zwei 2.dim. Koordinatensystemen.
 */


template<class TA, class TB> class Trans2 {
  
public:
  
  Trans2();
  void clear();
  
  void setA( const TA, const TA, const TA, const TA );
  void setB( const TB, const TB, const TB, const TB );
  
  void AtoB( const TA, const TA, TB &, TB & ) const;
  void BtoA( const TB, const TB, TA &, TA & ) const;
  
  int inA( const TA, const TA ) const;
  int inB( const TB, const TB ) const;
  
  void calc();
  
  Trans1<TA,TB> X, Y;
  
};



template<class TA, class TB> inline
Trans1<TA, TB>::Trans1()
{
  clear();
};


template<class TA, class TB> inline void
Trans1<TA, TB>::clear()
{
  flagA = flagB = 0;
}


template<class TA, class TB> inline void
Trans1<TA, TB>::setA( const TA nA1, const TA nA2 )
{
  A1 = nA1;
  A2 = nA2;
  
  flagA = 1;
  if( flagB ) calc();
}


template<class TA, class TB> inline void
Trans1<TA, TB>::setB( const TB nB1, const TB nB2 )
{
  B1 = nB1;
  B2 = nB2;
  
  flagB = 1;
  if( flagA ) calc();
}


template<class TA, class TB> inline void
Trans1<TA, TB>::AtoB( const TA A, TB &B ) const
{
  B = (TB)( Bm * A + Bc );
}


template<class TA, class TB> inline void
Trans1<TA, TB>::BtoA( const TB B, TA &A ) const
{
  A = (TA)( Am * B + Ac );
}


template<class TA, class TB> inline int
Trans1<TA, TB>::inA( const TA A ) const
{
  return A >= min( A1, A2 ) && A <= max( A1, A2 );
}


template<class TA, class TB> inline int
Trans1<TA, TB>::inB( const TB B ) const
{
  return B >= min( B1, B2 ) && B <= max( B1, B2 );
}


template<class TA, class TB> inline void
Trans1<TA, TB>::calc()
{
  Am = ( A2 - A1 ) / ( B2 - B1 );
  Ac = A1 - B1 * Am;
  
  Bm = ( B2 - B1 ) / ( A2 - A1 );
  Bc = B1 - A1 * Bm;
}



template<class TA, class TB> inline
Trans2<TA, TB>::Trans2()
{
  clear();
};


template<class TA, class TB> inline void
Trans2<TA, TB>::clear()
{
  X.clear();
  Y.clear();
}


template<class TA, class TB> inline void
Trans2<TA, TB>::setA( const TA nA1x, const TA nA1y, const TA nA2x, const TA nA2y )
{
  X.setA( nA1x, nA2x );
  Y.setA( nA1y, nA2y );
}


template<class TA, class TB> inline void
Trans2<TA, TB>::setB( const TB nB1x, const TB nB1y, const TB nB2x, const TB nB2y )
{
  X.setB( nB1x, nB2x );
  Y.setB( nB1y, nB2y );
}


template<class TA, class TB> inline void
Trans2<TA, TB>::AtoB( const TA Ax, const TA Ay, TB &Bx, TB &By ) const
{
  X.AtoB( Ax, Bx );
  Y.AtoB( Ay, By );
}


template<class TA, class TB> inline void
Trans2<TA, TB>::BtoA( const TB Bx, const TB By, TA &Ax, TA &Ay ) const
{
  X.BtoA( Bx, Ax );
  Y.BtoA( By, Ay );
}


template<class TA, class TB> inline int
Trans2<TA, TB>::inA( const TA Ax, const TA Ay ) const
{
  return X.inA( Ax ) && Y.inA( Ay );
}


template<class TA, class TB> inline int
Trans2<TA, TB>::inB( const TB Bx, const TB By ) const
{
  return X.inB( Bx ) && Y.inB( By );
}


template<class TA, class TB> inline void
Trans2<TA, TB>::calc()
{
  X.calc();
  Y.calc();
}


#endif
